@@ -172,9 +172,13 @@ class Client(object):
 
         self.include_paths = set(o.get('include_paths') or [])
         self.exclude_paths = set(o.get('exclude_paths') or [])
-        self.name = text_type(o.get('name') or o.get('machine') or defaults.NAME)
+        self.name = text_type(
+            o.get('name') or os.environ.get('SENTRY_NAME')
+            or o.get('machine') or defaults.NAME
+        )
         self.auto_log_stacks = bool(
-            o.get('auto_log_stacks') or defaults.AUTO_LOG_STACKS)
+            o.get('auto_log_stacks') or defaults.AUTO_LOG_STACKS
+        )
         self.capture_locals = bool(
             o.get('capture_locals', defaults.CAPTURE_LOCALS))
         self.string_max_length = int(
@@ -193,8 +197,11 @@ class Client(object):
             context = {'sys.argv': getattr(sys, 'argv', [])[:]}
         self.extra = context
         self.tags = o.get('tags') or {}
-        self.environment = o.get('environment') or None
-        self.release = o.get('release') or os.environ.get('HEROKU_SLUG_COMMIT')
+        self.environment = (
+            o.get('environment') or os.environ.get('SENTRY_ENVIRONMENT', None))
+        self.release = (
+            o.get('release') or os.environ.get('SENTRY_RELEASE')
+            or os.environ.get('HEROKU_SLUG_COMMIT'))
         self.repos = self._format_repos(o.get('repos'))
         self.sample_rate = (
             o.get('sample_rate')
@@ -430,12 +437,13 @@ class Client(object):
                     frame['in_app'] = False
                 else:
                     frame['in_app'] = (
-                        any(path.startswith(x) for x in self.include_paths) and
-                        not any(path.startswith(x) for x in self.exclude_paths)
+                        any(path.startswith(x) for x in self.include_paths)
+                        and not any(path.startswith(x) for x in self.exclude_paths)
                     )
 
+        transaction = None
         if not culprit:
-            culprit = self.transaction.peek()
+            transaction = self.transaction.peek()
 
         if not data.get('level'):
             data['level'] = kwargs.get('level') or logging.ERROR
@@ -460,7 +468,9 @@ class Client(object):
         if site:
             data['tags'].setdefault('site', site)
 
-        if culprit:
+        if transaction:
+            data['transaction'] = transaction
+        elif culprit:
             data['culprit'] = culprit
 
         if fingerprint:
@@ -615,7 +625,7 @@ class Client(object):
         :param stack: a stacktrace for the event
         :param tags: dict of extra tags
         :param sample_rate: a float in the range [0, 1] to sample this message
-        :return: a tuple with a 32-length string identifying this event
+        :return: a 32-length string identifying this event
         """
         if not self.is_enabled():
             return
